/* -*- C -*-
 * File: intrange.h
 *
 * Created: Thu Nov  4 11:20:36 2004
 *
 * $Id: newfile-template.el,v 1.1 2004/10/16 18:51:22 igor Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "intrange.h"


#define MAX_NUMBER_LENGTH 6


/* 
 * count_intrange 
 * precounts number of intervals
 * to allocate memory at once
 *
 * 
 */
int count_intrange (char* argstring)
{
  int count=0;
  int subcount=0;
  int char_len=0;
  char* orig_string=argstring;
  while (*argstring) {
    if (isdigit(*argstring)) {
      char_len++;
      if (1==char_len) {
	if (subcount==0) count++;
      }
      if (char_len>MAX_NUMBER_LENGTH) {
	fprintf(stderr,"intrange:too long number in %s:\n",orig_string);
	return -1;
      }
    } else {
      if (isspace(*argstring)) {
      } else if (',' == *argstring) {
	subcount=0;
	/* next interval */
      } else if ('-' == *argstring) {
	if (0==char_len && subcount==0) count++;
	subcount++;
      if (subcount>1) {
	fprintf(stderr,"intrange:too many - in %s:\n",orig_string);
	return -1;
      }
      } else {
      	fprintf(stderr,"intrange:strange charcater %c in %s:\n", *argstring, orig_string);
	return -1;
      }
      char_len=0;
    }
    argstring++;
  }
  if (0==count) {
      	fprintf(stderr,"intrange:empty string:\n");
  }
  return count;
}

int flash_buffer(char* buffer)
{
  int bufvalue=atoi(buffer);
  /* fprintf(stdout,"buffer=%d\n", bufvalue); */
  /* cleaning buffer for new char */
  int i;
  for(i = 0; i < MAX_NUMBER_LENGTH+1; buffer[i++] = 0);
  return bufvalue;
}

void finish_interval (int* char_len, int* subcount, char* buffer, struct intinterval* pair) {
  /* next interval */
  int num=flash_buffer(buffer);
  if (0==*subcount) {
    (*pair).min=num;
    (*pair).max=num;
  } else if (*char_len>0) {
    /* subcount ==1 */
    (*pair).max=num;
  } else {
    (*pair).max=INT_MAX;
  }
  *char_len=0;
  *subcount=0;
}

int fill_intrange (struct intinterval pairs[], char* argstring)
{
  int interval_count=0;
  int subcount=0;
  int char_len=0;
  char* orig_string=argstring;
  char buffer[MAX_NUMBER_LENGTH+1];
  memset(buffer, 0, sizeof(buffer));
  while (*argstring) {
    if (isdigit(*argstring)) {
      buffer[char_len]=*argstring;
      char_len++;
      if (char_len>MAX_NUMBER_LENGTH) {
	fprintf(stderr,"too long number in %s:\n",orig_string);
	return -1;
      }
    } else {
      if (isspace(*argstring)) {
      } else if (',' == *argstring) {
	finish_interval(&char_len, &subcount, buffer, &pairs[interval_count]);
	interval_count++;
      } else if ('-' == *argstring) {
	/* this interval */
	if (subcount==0) {
	  if (0==char_len) {
	    pairs[interval_count].min=INT_MIN;
	  } else {
	    int num=flash_buffer(buffer);
	    pairs[interval_count].min=num;
	    char_len=0;
	  }
	}
	subcount++;
      if (subcount>1) {
	fprintf(stderr,"too many - in %s:\n",orig_string);
	return -1;
      }
      } else {
      	fprintf(stderr,"strange charcater %c in %s:\n", *argstring, orig_string);
	return -1;
      }
      if (char_len>0) {
	/* cleaning buffer for new char */
	int i =flash_buffer(buffer);
	fprintf(stderr,"strange cleaning %d\n", i);
      }
      char_len=0;
    }
    argstring++;
  }
  finish_interval(&char_len, &subcount, buffer, &pairs[interval_count]);
  return ++interval_count;
}


struct intrange create_intrange (char* argstring)
{
  int intrange_presize;
  struct intrange irange;
  irange.pairs=NULL;
  irange.elnum=0;
  intrange_presize=count_intrange(argstring);
  if (-1==intrange_presize || 0==intrange_presize) {
    return irange;
  }
  irange.pairs=(struct intinterval (*)[] ) calloc(intrange_presize+1, sizeof(struct intinterval));
  if (! irange.pairs) {
    fprintf(stderr,"not enough memory for %s:\n",argstring);
    return irange;
  }
  irange.elnum=fill_intrange(*irange.pairs,argstring);
  return irange;
}

void print_intrange (struct intrange range) {
  int i;
  if (range.pairs) {
    for (i=0; i<range.elnum; i++) {
      printf("%d-%d\n",(*range.pairs)[i].min,(*range.pairs)[i].max);
    }
  } else {
    printf("empty range\n");
  }
}

int is_in_intrange(struct intrange range, int num) {
  int i;
  if (range.pairs) {
    for (i=0; i<range.elnum; i++) {
      if ((*range.pairs)[i].min<=num && (*range.pairs)[i].max>=num) return 1;
    }
  }
  return 0;
}

int is_empty_intrange(struct intrange range) {
  int i;
  if (range.pairs==NULL) return 1;
  return 0;
}

void drop_intrange(struct intrange range) {
  if (range.pairs) {
    free (range.pairs);
  }
}

/*  Local Variables:
 *  mode: C
 *  coding: cp1251
 *  End:
 */
